
from cuteSV.cuteSV_Description import Generation_VCF_header

def count_coverage(chr, s, e, f):
	read_count = set()
	for i in f.fetch(chr, s, e):
		read_count.add(i.query_name)
	return len(read_count)


def cal_GT(a, b):
	if b == 0:
		return "1/1"
	if a*1.0/b < 0.3:
		return "0/0"
	elif a*1.0/b >= 0.3 and a*1.0/b < 0.8:
		return "0/1"
	elif a*1.0/b >= 0.8 and a*1.0/b < 1.0:
		return "1/1"
	else:
		return "1/1"

def produce_GT(samfile, chr, bk, sup_read):
	search_start = max(bk - 20, 0)
	search_end = min(bk + 20, samfile.get_reference_length(chr))
	DP = count_coverage(chr, search_start, search_end, samfile)
	output_GT = "{FORMAT}\t{GT}:{DR}:{RE}".format(
		FORMAT = "GT:DR:DV", 
		GT = cal_GT(sup_read, DP),
		DR = max(DP - sup_read, 0),
		RE = sup_read)
	return output_GT


TriggerGT = {'False': 0, 'True': 1}


def generate_output(args, semi_result, contigINFO):
	
	'''
	Generation of VCF format file.
	VCF version: 4.2
	'''

	genotype_trigger = TriggerGT[args.genotype]

	svid = dict()
	svid["INS"] = 0
	svid["DEL"] = 0
	svid["BND"] = 0
	svid["DUP"] = 0
	svid["INV"] = 0

	file = open(args.output, 'w')
	Generation_VCF_header(file, contigINFO, args.sample)

	if genotype_trigger == 0:
		# without genotype
		for i in semi_result:
			if i[1] in ["DEL", "INS", "DUP", "INV"]:
				if i[1] == "INS":
					cal_end = int(i[2]) + 1
				else:
					cal_end = int(i[2]) + abs(int(float(i[3])))
				info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};BREAKPOINT_STD={BPSTD};SVLEN_STD={LENSTD};RE={RE}".format(
					PRECISION = "PRECISE", 
					SVTYPE = i[1], 
					SVLEN = i[3], 
					END = str(cal_end), 
					# BPSTD = i[5], 
					# LENSTD = i[6], 
					BPSTD = 0, 
					LENSTD = 0, 
					RE = i[4])
				file.write("{CHR}\t{POS}\t{ID}\tN\t{ALT}\t.\tPASS\t{INFO}\t{FORMAT}\t./.::{RE}\n".format(
					CHR = i[0], 
					POS = i[2], 
					ID = "cuteSV.%s.%d"%(i[1], svid[i[1]]),
					ALT = "<%s>"%(i[1]), 
					INFO = info_list, 
					FORMAT = "GT:DR:DV", 
					RE = i[4]))
				svid[i[1]] += 1

			else:
				# BND
				info_list = "{PRECISION};SVTYPE={SVTYPE};CHR2={CHR2};END={END};RE={RE}".format(
					PRECISION = "PRECISE", 
					SVTYPE = "BND", 
					CHR2 = i[3], 
					END = i[4], 
					RE = i[5])
				file.write("{CHR}\t{POS}\t{ID}\tN\t{ALT}\t.\tPASS\t{INFO}\t{FORMAT}\t./.::{RE}\n".format(
					CHR = i[0], 
					POS = i[2], 
					ID = "cuteSV.%s.%d"%("BND", svid["BND"]), 
					ALT = i[1], 
					INFO = info_list, 
					FORMAT = "GT:DR:DV", 
					RE = i[5]))
				svid["BND"] += 1
	else:
		# pass
		import pysam
		samfile = pysam.AlignmentFile(args.input)
		for i in semi_result:
			search_start = max(int(i[2]) - 20, 0)
			search_end = min(int(i[2]) + 20, samfile.get_reference_length(i[0]))
			DP = count_coverage(i[0], search_start, search_end, samfile)

			if i[1] in ["DEL", "INS", "DUP", "INV"]:
				if i[1] == "INS":
					cal_end = int(i[2]) + 1
				else:
					cal_end = int(i[2]) + abs(int(float(i[3])))
				info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};BREAKPOINT_STD={BPSTD};SVLEN_STD={LENSTD};RE={RE}".format(
					PRECISION = "PRECISE", 
					SVTYPE = i[1], 
					SVLEN = i[3], 
					END = str(cal_end), 
					# BPSTD = i[5], 
					# LENSTD = i[6], 
					BPSTD = 0, 
					LENSTD = 0, 
					RE = i[4])
				file.write("{CHR}\t{POS}\t{ID}\tN\t{ALT}\t.\tPASS\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}\n".format(
					CHR = i[0], 
					POS = i[2], 
					ID = "cuteSV.%s.%d"%(i[1], svid[i[1]]),
					ALT = "<%s>"%(i[1]), 
					INFO = info_list, 
					FORMAT = "GT:DR:DV", 
					GT = cal_GT(int(i[4]), DP), 
					DR = str(DP - int(i[4])), 
					RE = i[4]))
				svid[i[1]] += 1

			else:
				# BND
				info_list = "{PRECISION};SVTYPE={SVTYPE};CHR2={CHR2};END={END};RE={RE}".format(
					PRECISION = "PRECISE", 
					SVTYPE = "BND", 
					CHR2 = i[3], 
					END = i[4], 
					RE = i[5])
				file.write("{CHR}\t{POS}\t{ID}\tN\t{ALT}\t.\tPASS\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}\n".format(
					CHR = i[0], 
					POS = i[2], 
					ID = "cuteSV.%s.%d"%("BND", svid["BND"]), 
					ALT = i[1], 
					INFO = info_list, 
					FORMAT = "GT:DR:DV", 
					GT = cal_GT(int(i[5]), DP), 
					DR = str(DP - int(i[5])), 
					RE = i[5]))
				svid["BND"] += 1
		samfile.close()
	file.close()



import sys
import numpy as np
from collections import Counter

'''
*******************************************
				TO DO LIST
*******************************************
	1. Identify DP with samfile pointer;
	2. Add CIPOS, CILEN and/or CIEND;
	3. Determine (IM)PRECISE type.
*******************************************

'''

def resolution_DEL(path, chr, svtype, read_count, threshold_gloab, max_cluster_bias, 
	threshold_local, minimum_support_reads):

	'''
	cluster DEL
	********************************************************************************************
	path:	DEL.sigs
	chr:	chromosome id
	svtype:	<DEL>
	
	SEQTYPE		read_count 	max_cluster_bias 	sv_size		threshold_gloab 	threshold_local 
	--------------------------------------------------------------------------------------------
	CCS			3			200 bp (<500 bp)	30 bp 		0.4					0.5
	CLR			5/10		200 bp (<500 bp)	50 bp 		0.3					0.7
	--------------------------------------------------------------------------------------------
	
	Input file format
	--------------------------------------------------------------------------------------------
	column	#1	#2	#3	#4	#5
			DEL	CHR	BP	LEN	ID	
	#1	deletion type
	#2	chromosome number
	#3	breakpoint in each read
	#4	DEL_len in each read
	#5	read ID
	********************************************************************************************
	'''

	semi_del_cluster = list()
	semi_del_cluster.append([0,0,''])
	candidate_single_SV = list()

	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[1] != chr:
			continue

		pos = int(seq[2])
		indel_len = int(seq[3])
		read_id = seq[4]
		
		if pos - semi_del_cluster[-1][0] > max_cluster_bias:
			if len(semi_del_cluster) >= read_count:
				generate_del_cluster_2(semi_del_cluster, 
										chr, 
										svtype, 
										read_count, 
										threshold_gloab, 
										threshold_local, 
										minimum_support_reads, 
										candidate_single_SV)
			semi_del_cluster = []
			semi_del_cluster.append([pos, indel_len, read_id])
		else:
			semi_del_cluster.append([pos, indel_len, read_id])

	if len(semi_del_cluster) >= read_count:
		generate_del_cluster_2(semi_del_cluster, 
								chr, 
								svtype, 
								read_count, 
								threshold_gloab, 
								threshold_local, 
								minimum_support_reads, 
								candidate_single_SV)
	file.close()
	return candidate_single_SV

def generate_del_cluster_ccs(semi_del_cluster, chr, svtype, read_count, threshold_gloab,
 threshold_local, minimum_support_reads, candidate_single_SV):
	
	# calculate support reads
	support_read = len(list(set([i[2] for i in semi_del_cluster])))
	# print(support_read)
	if support_read < read_count:
		return

	del_cluster_len = sorted(semi_del_cluster, key = lambda x:x[1])

	last_len = del_cluster_len[0][1]
	temp_list = dict()
	temp_list[del_cluster_len[0][2]] = 0
	temp_count = 1
	subcluster_bp = list()
	subcluster_bp.append(del_cluster_len[0][0])
	# temp_LEN_sum = last_len
	subcluster_len = list()
	subcluster_len.append(last_len)

	predictions = list()
	for i in del_cluster_len[1:]:
		if last_len < 1000:
			bias = max(20, int(last_len*0.05))
		elif last_len < 5000:
			bias = max(50, int(last_len*0.02))
		elif last_len < 10000:
			bias = max(100,int(last_len*0.02))
		else:
			bias = 200
		if i[1] - last_len > bias:
			if temp_count >= read_count:
				if len(temp_list) >= read_count:
					# breakpoint = round(temp_sum / temp_count)
					breakpoint = Counter(subcluster_bp).most_common(1)[0]
					# del_len = round(temp_LEN_sum / temp_count)
					del_len = Counter(subcluster_len).most_common(1)[0]
					predictions.append([breakpoint, del_len, len(temp_list)])

			temp_list = dict()
			temp_count = 1
			subcluster_bp = list()
			subcluster_bp.append(i[0])
			subcluster_len = list()
			subcluster_len.append(i[1])
			temp_list[i[2]] = 0
		else:
			if i[2] not in temp_list:
				temp_list[i[2]] = 0
			else:
				temp_list[i[2]] += 1
			temp_count += 1
			subcluster_bp.append(i[0])
			subcluster_len.append(i[1])
		last_len = i[1]

	if temp_count >= read_count:
		if len(temp_list) >= read_count:
			breakpoint = Counter(subcluster_bp).most_common(1)[0]
			del_len = Counter(subcluster_len).most_common(1)[0]
			predictions.append([breakpoint, del_len, len(temp_list)])

	predictions = sorted(predictions, key = lambda x:-x[2])

	for i in predictions:
		if i[2] >= read_count and i[2] >= int(support_read / 3):
			if min(i[0][1], i[1][1]) * 2 >= i[2]:
				reliability = "PRECISION"
			else:
				reliability = "IMPRECISION"

			candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\t%s\n' % 
				(chr, svtype, i[0][0], -i[1][0], i[2], reliability))



def generate_del_cluster(semi_del_cluster, chr, svtype, read_count, 
	threshold_gloab, threshold_local, minimum_support_reads, candidate_single_SV):

	'''
	generate deletion
	*************************************************************
	threshold_gloab 	threshold_local 	minimum_support_reads
	-------------------------------------------------------------
		0.3					0.7 					5		CLR
		0.4					0.5 				  <=5		CCS
	*************************************************************
	'''

	# Remove duplicates
	read_tag = dict()
	for element in semi_del_cluster:
		if element[2] not in read_tag:
			read_tag[element[2]] = element
		else:
			if element[1] > read_tag[element[2]][1]:
				read_tag[element[2]] = element

	if len(read_tag) < read_count:
		return

	read_tag2SortedList = sorted(list(read_tag.values()), key = lambda x:x[1])
	global_len = [i[1] for i in read_tag2SortedList]
	DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP = threshold_gloab * np.mean(global_len)

	last_len = -DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP
	max_conut = 0
	max_LEN_sum = 0
	max_bps_sum = 0
	temp_count = 0
	temp_LEN_sum = 0
	temp_bps_sum = 0
	for i in read_tag2SortedList:
		if i[1] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP:
			if temp_count > max_conut:
				max_conut = temp_count
				max_LEN_sum = temp_LEN_sum
				max_bps_sum = temp_bps_sum
			temp_count = 1
			temp_LEN_sum = i[1]
			temp_bps_sum = i[0]
		else:
			temp_count += 1
			temp_LEN_sum += i[1]
			temp_bps_sum += i[0]
		last_len = i[1]
	if temp_count > max_conut:
		max_conut = temp_count
		max_LEN_sum = temp_LEN_sum
		max_bps_sum = temp_bps_sum

	breakpoint_starts = round(max_bps_sum / max_conut)
	signal_len = round(max_LEN_sum / max_conut)
	overlap_score = float(max_conut * 1.0 / len(read_tag))

	if max_conut >= minimum_support_reads and overlap_score >= threshold_local:
		# candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\t%d\t%.3f\n'%
		# 	(chr, svtype, breakpoint_starts, -signal_len, len(read_tag), max_conut, overlap_score))
		candidate_single_SV.append([chr, 
									svtype, 
									str(int(breakpoint_starts)), 
									str(-int(signal_len)), 
									str(len(read_tag))])

def generate_del_cluster_2(semi_del_cluster, chr, svtype, read_count, 
	threshold_gloab, threshold_local, minimum_support_reads, candidate_single_SV):

	'''
	generate deletion
	*************************************************************
	threshold_gloab 	threshold_local 	minimum_support_reads
	-------------------------------------------------------------
		0.3					0.7 					5		CLR
		0.4					0.5 				  <=5		CCS
	*************************************************************
	'''

	# Remove duplicates
	read_tag = dict()
	for element in semi_del_cluster:
		if element[2] not in read_tag:
			read_tag[element[2]] = element
		else:
			if element[1] > read_tag[element[2]][1]:
				read_tag[element[2]] = element

	if len(read_tag) < read_count:
		return

	read_tag2SortedList = sorted(list(read_tag.values()), key = lambda x:x[1])
	global_len = [i[1] for i in read_tag2SortedList]
	DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP = threshold_gloab * np.mean(global_len)

	last_len = read_tag2SortedList[0][1]

	alelle_collect = list()
	alelle_collect.append([[read_tag2SortedList[0][0]],[read_tag2SortedList[0][1]],[]])

	for i in read_tag2SortedList[1:]:
		if i[1] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP:
			alelle_collect[-1][2].append(len(alelle_collect[-1][0]))
			alelle_collect.append([[],[],[]])

		alelle_collect[-1][0].append(i[0])
		alelle_collect[-1][1].append(i[1])
		last_len = i[1]
	alelle_collect[-1][2].append(len(alelle_collect[-1][0]))
	alelle_sort = sorted(alelle_collect, key = lambda x:x[2])

	if alelle_sort[-1][2][0] >= minimum_support_reads and float(alelle_sort[-1][2][0] * 1.0 / len(read_tag)) >= threshold_local:
		breakpointStart = np.mean(alelle_sort[-1][0])
		breakpointStart_STD = np.std(alelle_sort[-1][0])
		signalLen = np.mean(alelle_sort[-1][1])
		signalLen_STD = np.std(alelle_sort[-1][1])
		# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-1][2][0], breakpointStart_STD, signalLen_STD)
		candidate_single_SV.append([chr, 
									svtype, 
									str(int(breakpointStart)), 
									str(int(-signalLen)), 
									str(alelle_sort[-1][2][0]), 
									"%.3f"%breakpointStart_STD, 
									"%.3f"%signalLen_STD])

		# extend to next alelle
		if (len(alelle_sort) > 1 and alelle_sort[-2][2][0] >= minimum_support_reads 
			and alelle_sort[-2][2][0] + alelle_sort[-1][2][0] >= 0.95*len(read_tag) 
			and alelle_sort[-2][2][0] >= 0.3*len(read_tag)):
			breakpointStart = np.mean(alelle_sort[-2][0])
			breakpointStart_STD = np.std(alelle_sort[-2][0])
			signalLen = np.mean(alelle_sort[-2][1])
			last_signalLen_STD = signalLen_STD
			signalLen_STD = np.std(alelle_sort[-2][1])
			# if signalLen_STD < last_signalLen_STD:
			# 	# pass
			# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-2][2][0], breakpointStart_STD, signalLen_STD)
			candidate_single_SV.append([chr, 
										svtype, 
										str(int(breakpointStart)), 
										str(int(-signalLen)), 
										str(alelle_sort[-2][2][0]), 
										"%.3f"%breakpointStart_STD, 
										"%.3f"%signalLen_STD])

	elif alelle_sort[-2][2][0] >= minimum_support_reads and alelle_sort[-2][2][0] + alelle_sort[-1][2][0] >= 0.95*len(read_tag):
		if alelle_sort[-2][2][0] >= 0.4*len(read_tag):
			breakpointStart = np.mean(alelle_sort[-1][0])
			breakpointStart_STD = np.std(alelle_sort[-1][0])
			signalLen = np.mean(alelle_sort[-1][1])
			signalLen_STD = np.std(alelle_sort[-1][1])
			# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-1][2][0], breakpointStart_STD, signalLen_STD)
			candidate_single_SV.append([chr, 
										svtype, 
										str(int(breakpointStart)), 
										str(int(-signalLen)), 
										str(alelle_sort[-1][2][0]), 
										"%.3f"%breakpointStart_STD, 
										"%.3f"%signalLen_STD])

			breakpointStart = np.mean(alelle_sort[-2][0])
			breakpointStart_STD = np.std(alelle_sort[-2][0])
			signalLen = np.mean(alelle_sort[-2][1])
			signalLen_STD = np.std(alelle_sort[-2][1])
			# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-2][2][0], breakpointStart_STD, signalLen_STD)
			candidate_single_SV.append([chr, 
										svtype, 
										str(int(breakpointStart)), 
										str(int(-signalLen)), 
										str(alelle_sort[-2][2][0]), 
										"%.3f"%breakpointStart_STD, 
										"%.3f"%signalLen_STD])


	

def resolution_INS(path, chr, svtype, read_count, threshold_gloab, 
	max_cluster_bias, threshold_local, minimum_support_reads):
	
	'''
	cluster INS
	********************************************************************************************
	path:	INS.sigs
	chr:	chromosome id
	svtype:	<INS>
	
	SEQTYPE		read_count 	max_cluster_bias 	sv_size		threshold_gloab 	threshold_local 
	--------------------------------------------------------------------------------------------
	CCS			3			200 bp (<500 bp)	30 bp 		0.65				0.7
	CLR			5/10		100 bp (<500 bp)	50 bp 		0.2					0.6
	--------------------------------------------------------------------------------------------
	
	Input file format
	--------------------------------------------------------------------------------------------
	column	#1	#2	#3	#4	#5
			INS	CHR	BP	LEN	ID	
	#1	insertion type
	#2	chromosome number
	#3	breakpoint in each read
	#4	DEL_len in each read
	#5	read ID
	********************************************************************************************
	'''

	semi_ins_cluster = list()
	semi_ins_cluster.append([0,0,''])
	candidate_single_SV = list()

	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[1] != chr:
			continue

		pos = int(seq[2])
		indel_len = int(seq[3])
		read_id = seq[4]
		
		if pos - semi_ins_cluster[-1][0] > max_cluster_bias:
			if len(semi_ins_cluster) >= read_count:
				generate_ins_cluster_2(semi_ins_cluster, 
										chr, 
										svtype, 
										read_count, 
										threshold_gloab, 
										threshold_local, 
										minimum_support_reads, 
										candidate_single_SV)
			semi_ins_cluster = []
			semi_ins_cluster.append([pos, indel_len, read_id])
		else:
			semi_ins_cluster.append([pos, indel_len, read_id])

	if len(semi_ins_cluster) >= read_count:
		generate_ins_cluster_2(semi_ins_cluster, 
								chr, 
								svtype, 
								read_count, 
								threshold_gloab, 
								threshold_local, 
								minimum_support_reads, 
								candidate_single_SV)
	file.close()
	return candidate_single_SV

def generate_ins_cluster_ccs(semi_ins_cluster, chr, svtype, read_count, 
	threshold_gloab, threshold_local, minimum_support_reads, candidate_single_SV):

	# calculate support reads
	support_read = len(list(set([i[2] for i in semi_ins_cluster])))
	if support_read < read_count:
		return

	del_cluster_len = sorted(semi_ins_cluster, key = lambda x:x[1])

	last_len = del_cluster_len[0][1]
	temp_list = dict()
	temp_list[del_cluster_len[0][2]] = 0
	temp_count = 1
	subcluster_bp = list()
	subcluster_bp.append(del_cluster_len[0][0])
	# temp_LEN_sum = last_len
	subcluster_len = list()
	subcluster_len.append(last_len)

	predictions = list()
	for i in del_cluster_len[1:]:
		if last_len < 1000:
			bias = max(20, int(last_len*0.05))
		elif last_len < 5000:
			bias = max(50, int(last_len*0.02))
		elif last_len < 10000:
			bias = max(100,int(last_len*0.02))
		else:
			bias = 200
		if i[1] - last_len > bias:
			if temp_count >= read_count:
				if len(temp_list) >= read_count:
					# breakpoint = round(temp_sum / temp_count)
					breakpoint = Counter(subcluster_bp).most_common(1)[0]
					# del_len = round(temp_LEN_sum / temp_count)
					del_len = Counter(subcluster_len).most_common(1)[0]
					predictions.append([breakpoint, del_len, len(temp_list)])

			temp_list = dict()
			temp_count = 1
			subcluster_bp = list()
			subcluster_bp.append(i[0])
			subcluster_len = list()
			subcluster_len.append(i[1])
			temp_list[i[2]] = 0
		else:
			if i[2] not in temp_list:
				temp_list[i[2]] = 0
			else:
				temp_list[i[2]] += 1
			temp_count += 1
			subcluster_bp.append(i[0])
			subcluster_len.append(i[1])
		last_len = i[1]

	if temp_count >= read_count:
		if len(temp_list) >= read_count:
			breakpoint = Counter(subcluster_bp).most_common(1)[0]
			del_len = Counter(subcluster_len).most_common(1)[0]
			predictions.append([breakpoint, del_len, len(temp_list)])

	predictions = sorted(predictions, key = lambda x:-x[2])

	for i in predictions:
		# print(i)
		# print(support_read)
		# if i[2] >= read_count and i[2] >= int(support_read / 3):
		if i[2] >= read_count:
			if min(i[0][1], i[1][1]) * 2 >= i[2]:
				reliability = "PRECISION"
			else:
				reliability = "IMPRECISION"

			if i[0][1] == 1 and i[1][1] == 1:
				return

			candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\t%s\n' % 
				(chr, svtype, i[0][0], i[1][0], i[2], reliability))


def generate_ins_cluster(semi_ins_cluster, chr, svtype, read_count, 
	threshold_gloab, threshold_local, minimum_support_reads, candidate_single_SV):
		
	'''
	generate deletion
	*************************************************************
	threshold_gloab 	threshold_local 	minimum_support_reads
	-------------------------------------------------------------
		0.2					0.6 					5		CLR
		0.65				0.7 				  <=5		CCS
	*************************************************************
	'''

	# Remove duplicates
	read_tag = dict()
	for element in semi_ins_cluster:
		if element[2] not in read_tag:
			read_tag[element[2]] = element
		else:
			if element[1] > read_tag[element[2]][1]:
				read_tag[element[2]] = element

	if len(read_tag) < read_count:
		return

	read_tag2SortedList = sorted(list(read_tag.values()), key = lambda x:x[1])
	# start&end breakpoint
	breakpoint_starts = [i[0] for i in read_tag2SortedList]
	global_len = [i[1] for i in read_tag2SortedList]

	DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP = threshold_gloab * np.mean(global_len)

	last_len = -DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP
	max_conut = 0
	max_LEN_sum = 0
	temp_count = 0
	temp_LEN_sum = 0
	for i in read_tag2SortedList:
		if i[1] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP:
			if temp_count > max_conut:
				max_conut = temp_count
				max_LEN_sum = temp_LEN_sum
			temp_count = 1
			temp_LEN_sum = i[1]
		else:
			temp_count += 1
			temp_LEN_sum += i[1]
		last_len = i[1]
	if temp_count > max_conut:
		max_conut = temp_count
		max_LEN_sum = temp_LEN_sum

	breakpointStart = np.mean(breakpoint_starts)
	breakpointStart_STD = np.std(breakpoint_starts)
	signal_len = round(max_LEN_sum / max_conut)
	overlap_score = float(max_conut * 1.0 / len(read_tag))

	if max_conut >= minimum_support_reads and overlap_score >= threshold_local:
		# candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\t%d\t%.3f\t%.3f\t%d\n' % 
		# 	(chr, svtype, breakpointStart, signal_len, len(read_tag), max_conut, overlap_score, breakpointStart_STD, np.mean(global_len)))
		candidate_single_SV.append([chr, 
									svtype, 
									str(int(breakpointStart)), 
									str(int(signal_len)), 
									str(len(read_tag))])

def generate_ins_cluster_2(semi_ins_cluster, chr, svtype, read_count, 
	threshold_gloab, threshold_local, minimum_support_reads, candidate_single_SV):
		
	'''
	generate deletion
	*************************************************************
	threshold_gloab 	threshold_local 	minimum_support_reads
	-------------------------------------------------------------
		0.2					0.6 					5		CLR
		0.65				0.7 				  <=5		CCS
	*************************************************************
	'''

	# Remove duplicates
	read_tag = dict()
	for element in semi_ins_cluster:
		if element[2] not in read_tag:
			read_tag[element[2]] = element
		else:
			if element[1] > read_tag[element[2]][1]:
				read_tag[element[2]] = element

	if len(read_tag) < read_count:
		return

	read_tag2SortedList = sorted(list(read_tag.values()), key = lambda x:x[1])
	# start&end breakpoint
	global_len = [i[1] for i in read_tag2SortedList]
	DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP = threshold_gloab * np.mean(global_len)

	last_len = read_tag2SortedList[0][1]

	alelle_collect = list()
	alelle_collect.append([[read_tag2SortedList[0][0]],[read_tag2SortedList[0][1]],[]])

	for i in read_tag2SortedList[1:]:
		if i[1] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP:
			alelle_collect[-1][2].append(len(alelle_collect[-1][0]))
			alelle_collect.append([[],[],[]])

		alelle_collect[-1][0].append(i[0])
		alelle_collect[-1][1].append(i[1])
		last_len = i[1]
	alelle_collect[-1][2].append(len(alelle_collect[-1][0]))
	alelle_sort = sorted(alelle_collect, key = lambda x:x[2])

	# print(len(read_tag), DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP)
	# for i in alelle_sort:
	# 	for j in i:
	# 		print(j)

	# print(alelle_sort[-1][2][0], minimum_support_reads)
	# print(float(alelle_sort[-1][2][0] * 1.0 / len(read_tag)), threshold_local)

	if alelle_sort[-1][2][0] >= minimum_support_reads and float(alelle_sort[-1][2][0] * 1.0 / len(read_tag)) >= threshold_local:
		breakpointStart = np.mean(alelle_sort[-1][0])
		breakpointStart_STD = np.std(alelle_sort[-1][0])
		signalLen = np.mean(alelle_sort[-1][1])
		signalLen_STD = np.std(alelle_sort[-1][1])
		# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-1][2][0], breakpointStart_STD, signalLen_STD)
		candidate_single_SV.append([chr, 
									svtype, 
									str(int(breakpointStart)), 
									str(int(signalLen)), 
									str(alelle_sort[-1][2][0]), 
									"%.3f"%breakpointStart_STD, 
									"%.3f"%signalLen_STD])

		# extend to next alelle
		if (len(alelle_sort) > 1 and alelle_sort[-2][2][0] >= minimum_support_reads 
			and alelle_sort[-2][2][0] + alelle_sort[-1][2][0] >= 0.95*len(read_tag) 
			and alelle_sort[-2][2][0] >= 0.3*len(read_tag)):
			breakpointStart = np.mean(alelle_sort[-2][0])
			breakpointStart_STD = np.std(alelle_sort[-2][0])
			signalLen = np.mean(alelle_sort[-2][1])
			last_signalLen_STD = signalLen_STD
			signalLen_STD = np.std(alelle_sort[-2][1])
			if signalLen_STD < last_signalLen_STD:
				# pass
				# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-2][2][0], breakpointStart_STD, signalLen_STD)
				candidate_single_SV.append([chr, 
											svtype, 
											str(int(breakpointStart)), 
											str(int(signalLen)), 
											str(alelle_sort[-2][2][0]), 
											"%.3f"%breakpointStart_STD, 
											"%.3f"%signalLen_STD])

	elif alelle_sort[-2][2][0] >= minimum_support_reads and alelle_sort[-2][2][0] + alelle_sort[-1][2][0] >= 0.95*len(read_tag):
		if alelle_sort[-2][2][0] >= 0.4*len(read_tag):
			breakpointStart = np.mean(alelle_sort[-1][0])
			breakpointStart_STD = np.std(alelle_sort[-1][0])
			signalLen = np.mean(alelle_sort[-1][1])
			signalLen_STD = np.std(alelle_sort[-1][1])
			# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-1][2][0], breakpointStart_STD, signalLen_STD)
			candidate_single_SV.append([chr, 
										svtype, 
										str(int(breakpointStart)), 
										str(int(signalLen)), 
										str(alelle_sort[-1][2][0]), 
										"%.3f"%breakpointStart_STD, 
										"%.3f"%signalLen_STD])

			breakpointStart = np.mean(alelle_sort[-2][0])
			breakpointStart_STD = np.std(alelle_sort[-2][0])
			signalLen = np.mean(alelle_sort[-2][1])
			signalLen_STD = np.std(alelle_sort[-2][1])
			# print(chr, svtype, int(breakpointStart), int(signalLen), alelle_sort[-2][2][0], breakpointStart_STD, signalLen_STD)
			candidate_single_SV.append([chr, 
										svtype, 
										str(int(breakpointStart)), 
										str(int(signalLen)), 
										str(alelle_sort[-2][2][0]), 
										"%.3f"%breakpointStart_STD, 
										"%.3f"%signalLen_STD])

def run_del(args):
	return resolution_DEL(*args)

def run_ins(args):
	return resolution_INS(*args)



import sys
import numpy as np
from collections import Counter

'''
*******************************************
				TO DO LIST
*******************************************
	1. Identify DP with samfile pointer;
	2. Add CIPOS, CILEN and/or CIEND;
	3. Determine (IM)PRECISE type.
	4. Filter DUP to improve INS FN rate.
*******************************************
'''

def resolution_DUP(path, chr, read_count, max_cluster_bias, sv_size):
	semi_dup_cluster = list()
	semi_dup_cluster.append([0,0,''])
	candidate_single_SV = list()

	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[1] != chr:
			continue

		pos_1 = int(seq[2])
		pos_2 = int(seq[3])
		read_id = seq[4]
		
		if pos_1 - semi_dup_cluster[-1][0] > max_cluster_bias:
			if len(semi_dup_cluster) >= read_count:
				generate_dup_cluster_ccs(semi_dup_cluster, 
										chr, 
										read_count, 
										max_cluster_bias, 
										sv_size, 
										candidate_single_SV)
			semi_dup_cluster = []
			semi_dup_cluster.append([pos_1, pos_2, read_id])
		else:
			semi_dup_cluster.append([pos_1, pos_2, read_id])

	if len(semi_dup_cluster) >= read_count:
		generate_dup_cluster_ccs(semi_dup_cluster, 
								chr, read_count, 
								max_cluster_bias, 
								sv_size, 
								candidate_single_SV)
	file.close()
	return candidate_single_SV

def generate_dup_cluster_ccs(semi_dup_cluster, chr, read_count, max_cluster_bias, sv_size, candidate_single_SV):
	# calculate support reads
	support_read = len(list(set([i[2] for i in semi_dup_cluster])))
	# print(support_read)
	if support_read < read_count:
		return

	dup_cluster_len = sorted(semi_dup_cluster, key = lambda x:x[1])
	# for i in dup_cluster_len:
	# 	print(i)

	last_len = dup_cluster_len[0][1]
	temp_list = dict()
	temp_list[dup_cluster_len[0][2]] = 0
	temp_count = 1
	subcluster_bp1 = list()
	subcluster_bp1.append(dup_cluster_len[0][0])
	# temp_LEN_sum = last_len
	subcluster_bp2 = list()
	subcluster_bp2.append(last_len)

	predictions = list()
	for i in dup_cluster_len[1:]:
		if i[1] - last_len > max_cluster_bias:
			if temp_count >= read_count:
				if len(temp_list) >= read_count:
					# breakpoint = round(temp_sum / temp_count)
					breakpoint_1 = Counter(subcluster_bp1).most_common(1)[0]
					# del_len = round(temp_LEN_sum / temp_count)
					breakpoint_2 = Counter(subcluster_bp2).most_common(1)[0]
					predictions.append([breakpoint_1, breakpoint_2, len(temp_list)])
					# print(breakpoint_1, breakpoint_2, len(temp_list))

			temp_list = dict()
			temp_count = 1
			subcluster_bp1 = list()
			subcluster_bp1.append(i[0])
			subcluster_bp2 = list()
			subcluster_bp2.append(i[1])
			temp_list[i[2]] = 0
		else:
			if i[2] not in temp_list:
				temp_list[i[2]] = 0
			else:
				temp_list[i[2]] += 1
			temp_count += 1
			subcluster_bp1.append(i[0])
			subcluster_bp2.append(i[1])
		last_len = i[1]

	if temp_count >= read_count:
		if len(temp_list) >= read_count:
			breakpoint_1 = Counter(subcluster_bp1).most_common(1)[0]
			breakpoint_2 = Counter(subcluster_bp2).most_common(1)[0]
			predictions.append([breakpoint_1, breakpoint_2, len(temp_list)])

	predictions = sorted(predictions, key = lambda x:-x[2])

	for i in predictions:
		# print(i)
		if i[2] >= read_count and i[2] >= int(support_read / 3) and i[1][0] - i[0][0] >= sv_size:
			if min(i[0][1], i[1][1]) * 2 >= i[2]:
				reliability = "PRECISION"
			else:
				reliability = "IMPRECISION"

			# candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\t%s\n' % (chr, 'DUP', i[0][0], i[1][0] - i[0][0], i[2], reliability))
			candidate_single_SV.append([chr, 'DUP', str(i[0][0]), str(i[1][0] - i[0][0]), str(i[2])])


def generate_semi_dup_cluster(semi_dup_cluster, chr, read_count, max_cluster_bias, sv_size, dup_candidates, candidate_single_SV):
	'''
	generate DUP
	*******************************************
	overlap_size 	max_cluster_bias 	sv_size
	-------------------------------------------
	0.5				50 bp (<500 bp)		50 bp
	*******************************************
	'''
	start = list()
	end = list()
	start_id = list()
	end_id = list()
	read_tag = dict()
	for element in semi_dup_cluster:
		start.append(element[0])
		end.append(element[1])
		start_id.append([element[0], element[2]])
		end_id.append([element[1], element[2]])
		# if element[2] not in read_tag:
		# 	read_tag[element[2]] = 0

	# re_l, rl_id = acquire_locus(int(min(start)/1000)*1000, int(max(start)/1000+1)*1000, "l_DUP", chr, dup_candidates)
	# re_r, rr_id = acquire_locus(min(end)-1000, max(end)+1000, "DUP_r", chr, dup_candidates)
	re_l = acquire_locus(min(start)-max_cluster_bias, max(start)+max_cluster_bias, "l_DUP", chr, dup_candidates)
	re_r = acquire_locus(min(end)-max_cluster_bias, max(end)+max_cluster_bias, "DUP_r", chr, dup_candidates)
	info_1 = cal_cluster_breakpoint(start_id+re_l, max_cluster_bias)
	info_2 = cal_cluster_breakpoint(end_id+re_r, max_cluster_bias)

	cal_pos = 0
	for i in info_1:
		if i[1] not in read_tag:
			read_tag[i[1]] = 0
		cal_pos += i[0]
	breakpoint_1 = int(cal_pos/len(info_1))
	cal_pos = 0
	for i in info_2:
		if i[1] not in read_tag:
			read_tag[i[1]] = 0
		cal_pos += i[0]
	breakpoint_2 = int(cal_pos/len(info_2))	

	if breakpoint_2 - breakpoint_1 + 1 >= sv_size and len(read_tag) >= read_count:
		# print info_1
		# print info_2
		# print("%s\t%s\t%d\t%d\t%d"%(chr, "DUP", breakpoint_1, breakpoint_2-breakpoint_1+1, len(read_tag)))
		# candidate_single_SV.append("%s\t%s\t%d\t%d\t%d\n"%(chr, "DUP", breakpoint_1, breakpoint_2-breakpoint_1+1, len(read_tag)))
		candidate_single_SV.append([chr, "DUP", str(breakpoint_1), str(breakpoint_2-breakpoint_1+1), str(len(read_tag))])

def acquire_locus(down, up, keytype, chr, MainCandidate):
	re = list()
	# re_id = list()
	if chr not in MainCandidate[keytype]:
		return re
	if int(up/10000) == int(down/10000):
		key_1 = int(down/10000)
		if key_1 not in MainCandidate[keytype][chr]:
			return re
		for i in range(int((up%10000)/50)-int((down%10000)/50)+1):
			# exist a bug ***********************************
			key_2 = int((down%10000)/50)+i
			if key_2 not in MainCandidate[keytype][chr][key_1]:
				continue
			for ele in MainCandidate[keytype][chr][key_1][key_2]:
				if ele[0] >= down and ele[0] <= up:
					re.append([ele[0],ele[1]])
					# re_id.append(ele[1])
	else:
		key_1 = int(down/10000)
		if key_1 in MainCandidate[keytype][chr]:
			for i in range(200-int((down%10000)/50)):
				# exist a bug ***********************************
				key_2 = int((down%10000)/50)+i
				if key_2 not in MainCandidate[keytype][chr][key_1]:
					continue
				for ele in MainCandidate[keytype][chr][key_1][key_2]:
					if ele[0] >= down and ele[0] <= up:
						re.append([ele[0],ele[1]])
						# re_id.append(ele[1])
		key_1 += 1
		if key_1 not in MainCandidate[keytype][chr]:
			return re
		for i in range(int((up%10000)/50)+1):
			# exist a bug ***********************************
			key_2 = i
			if key_2 not in MainCandidate[keytype][chr][key_1]:
				continue
			for ele in MainCandidate[keytype][chr][key_1][key_2]:
				if ele[0] >= down and ele[0] <= up:
					re.append([ele[0],ele[1]])
					# re_id.append(ele[1])
	return re

def cal_cluster_breakpoint(pos_list, bias):
	pos_list = sorted(pos_list, key = lambda x:x[0])
	#print pos_list
	temp = list()
	temp.append(pos_list[0])
	best_bp =list()
	len_best_bp = 0
	for pos in pos_list[1:]:
		if temp[-1][0] + bias < pos[0]:
			if len(temp) > len_best_bp:
				len_best_bp = len(temp)
				# best_bp = sum(temp)/len(temp)
				best_bp = temp
			temp = list()
			temp.append(pos)
		else:
			temp.append(pos)
	if len(temp) > len_best_bp:
		len_best_bp = len(temp)
		# best_bp = sum(temp)/len(temp)
		best_bp = temp
	return best_bp

def run_dup(args):
	return resolution_DUP(*args)



import sys
import argparse
import logging
import time

def load_high_conf_bed(path):
	confbed = dict()
	file = open(path,'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		chr = seq[0]
		svtype = seq[1]
		istart = int(seq[2])
		iend = int(seq[3])

		if svtype not in confbed:
			confbed[svtype] = dict()
		if chr not in confbed[svtype]:
			confbed[svtype][chr] = list()
		confbed[svtype][chr].append([istart, iend, 0, 0, 0])
	file.close()
	return confbed

def pase_info(seq):
	info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0, "CHR2": ''}
	for i in seq.split(';'):
		if i.split('=')[0] in ["SVLEN", "END", "RE"]:
			try:
				info[i.split('=')[0]] = abs(int(float(i.split('=')[1])))
			except:
				pass
		if i.split('=')[0] in ["SVTYPE", "CHR2"]:
			info[i.split('=')[0]] = i.split('=')[1]
	return info

def load_callset_cuteSV(path, filter):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		ALT = seq[2][7:10]
		# if ALT == "DUP":
		# 	ALT = "INS"
		if ALT not in ["INS", "INV", "DEL", "DUP", "BND"]:
			continue

		info = pase_info(seq[7])

		if ALT not in callset:
			callset[ALT] = dict()
		if chr not in callset[ALT]:
			if ALT == 'BND':
				callset[ALT][chr] = dict()
			else:
				callset[ALT][chr] = list()

		if ALT == "BND":
			if info["CHR2"] not in callset[ALT][chr]:
				callset[ALT][chr][info["CHR2"]] = list()
			# if info["RE"] >= filter:
			callset[ALT][chr][info["CHR2"]].append([pos, info["END"], 0])
		else:
			# if info["SVLEN"] >= 50:
			if info["SVLEN"] >= 50 and info["SVLEN"] <= 100000:
				callset[ALT][chr].append([pos, info["SVLEN"], info["END"], 0])
	file.close()
	return callset

def eva_record(call_A, call_B, bias, offect):
	for svtype in call_A:
		if svtype not in call_B:
			continue
		else:
			if svtype == "BND":
				for chr1 in call_A[svtype]:
					if chr1 not in call_B[svtype]:
						continue
					else:
						for chr2 in call_A[svtype][chr1]:
							if chr2 not in call_B[svtype][chr1]:
								continue
							else:
								for i in call_A[svtype][chr1][chr2]:
									for j in call_B[svtype][chr1][chr2]:
										if abs(i[0] - j[0]) <= offect and abs(i[1] - j[1]) <= offect:
											i[2] += 1
											j[2] += 1
										else:
											pass
			else:
				for chr in call_A[svtype]:
					if chr not in call_B[svtype]:
						continue
					else:
						for i in call_A[svtype][chr]:
							for j in call_B[svtype][chr]:
								if i[0] - offect <= j[0] <= i[2] + offect or i[0] - offect <= j[2] <= i[2] + offect or j[0] - offect <= i[0] <= j[2] + offect:
									if min(i[1], j[1])*1.0/max(i[1], j[1]) >= bias:
										i[3] += 1
										j[3] += 1
									else:
										pass

def statistics_true_possitive(callset, SVTYPE="ALL"):
	record = 0
	non_record = 0
	if SVTYPE == "ALL":
		for svtype in callset:
			if svtype == "BND":
				for chr1 in callset[svtype]:
					for chr2 in callset[svtype][chr1]:
						for res in callset[svtype][chr1][chr2]:
							record += 1
							if res[2] == 0:
								non_record += 1
			else:
				for chr in callset[svtype]:
					for res in callset[svtype][chr]:
						record += 1
						if res[3] == 0:
							non_record += 1
	else:
		if SVTYPE == "BND":
			for chr1 in callset[SVTYPE]:
				for chr2 in callset[SVTYPE][chr1]:
					for res in callset[SVTYPE][chr1][chr2]:
						record += 1
						if res[2] == 0:
							non_record += 1
		else:
			for chr in callset[SVTYPE]:
				for res in callset[SVTYPE][chr]:
					record += 1
					if res[3] == 0:
						non_record += 1
	return record, non_record

def eva_hc(pac, ont, hc, bias, offect):
	for chr in hc['INS']:
		for ele in hc["INS"][chr]:
			if chr not in pac["INS"]:
				pass
			else:
				for i in pac["INS"][chr]:
					if ele[0] - offect <= i[0] <= ele[1] + offect:
						ele[2] = 1
					else:
						pass
			if chr in ont["INS"]:
				for j in ont["INS"][chr]:
					if ele[0] - offect <= j[0] <= ele[1] + offect:
						ele[3] = 1
					else:
						pass

	for chr in hc['DEL']:
		for ele in hc['DEL'][chr]:
			if chr in pac['DEL']:
				for i in pac['DEL'][chr]:
					if ele[0] - offect <= i[0] <= ele[1] + offect or ele[0] - offect <= i[2] <= ele[1] + offect or i[0] - offect <= ele[0] <= i[2] + offect:
						if min(ele[1]-ele[0], i[1])*1.0/max(ele[1]-ele[0], i[1]) >= bias:
							ele[2] = 1
			if chr in ont['DEL']:
				for i in ont['DEL'][chr]:
					if ele[0] - offect <= i[0] <= ele[1] + offect or ele[0] - offect <= i[2] <= ele[1] + offect or i[0] - offect <= ele[0] <= i[2] + offect:
						if min(ele[1]-ele[0], i[1])*1.0/max(ele[1]-ele[0], i[1]) >= bias:
							ele[3] = 1

	for SVTYPE in ["INS", 'DEL']:
		hc_pac = 0
		hc_ont = 0
		hc_pac_ont = 0
		hc_num = 0
		for chr in hc[SVTYPE]:
			for i in hc[SVTYPE][chr]:
				hc_num += 1
				if i[2] == 1:
					hc_pac += 1
				if i[3] == 1:
					hc_ont += 1
				if i[2]+i[3] == 2:
					i[4] = 1
					hc_pac_ont += 1
		logging.info("%s\thc:%d\tPacBio_hc:%d\tONT_hc:%d\tPacBio_ONT_hc:%d."%(SVTYPE, hc_num, hc_pac, hc_ont, hc_pac_ont))

def main_ctrl(args):	
	pac = load_callset_cuteSV(args.C1, 10)
	ont = load_callset_cuteSV(args.C2, 5)
	hc = load_high_conf_bed(args.C3)
	# for SVTYPE in hc:
	# 	for chr in hc[SVTYPE]:
	# 		for i in hc[SVTYPE][chr]:
	# 			print(SVTYPE, chr, i)
	logging.info("Calculate overlaps.")
	eva_record(pac, ont, args.bias, args.offect)
	eva_hc(pac, ont, hc, args.bias, args.offect)
	svtype = ["DEL", "DUP", "INS", "INV", "BND", "ALL"]
	for i in svtype:
		pac_r, pac_nol = statistics_true_possitive(pac, i)
		pac_ol = pac_r - pac_nol
		ont_r, ont_nol = statistics_true_possitive(ont, i)
		ont_ol = ont_r - ont_nol
		logging.info("%s\tPacBio:%d %d %.4f\tONT:%d %d %.4f."%(i, pac_r, pac_ol, 1.0*pac_ol/pac_r, ont_r, ont_ol, 1.0*ont_ol/ont_r))


def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Evaluate SV callset generated by cuteSV
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="Trio_eval", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("C1", type=str, help="PacBio callset")
	parser.add_argument('C2', type=str, help = "ONT callset")
	parser.add_argument('C3', type=str, help = "High confidence callset")
	parser.add_argument('-b', '--bias', help = "Bias of overlaping.[%(default)s]", default = 0.7, type = float)
	parser.add_argument('-o', '--offect', help = "Offect of translocation overlaping.[%(default)s]", default = 1000, type = int)
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])


import sys
import argparse
import logging
import time

def load_high_conf_bed(path):
	confbed = dict()
	file = open(path,'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		chr = seq[0]
		istart = int(seq[1])
		iend = int(seq[2])
		if chr not in confbed:
			confbed[chr] = list()
		confbed[chr].append([istart, iend])
	file.close()
	return confbed

def judge_bed(a, b, clist):
	for i in clist:
		if a <= i[0] and b >= i[0]:
			return 1
		if i[0] >= a and a <= i[1]:
			return 1
	return 0

def parse_BND(tag):
	if len(tag.split('[')) == 1:
		return tag.split(']')[1].split(':')[0], int(tag.split(']')[1].split(':')[1])
	else:
		return tag.split('[')[1].split(':')[0], int(tag.split('[')[1].split(':')[1])

def pase_info(seq):
	info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0, "CHR2": ''}
	for i in seq.split(';'):
		if i.split('=')[0] in ["SVLEN", "END", "RE"]:
			try:
				info[i.split('=')[0]] = abs(int(i.split('=')[1]))
			except:
				pass
		if i.split('=')[0] in ["SVTYPE", "CHR2"]:
			info[i.split('=')[0]] = i.split('=')[1]
	return info

def load_callset_cuteSV(path, filter, confbed):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		ALT = seq[2][7:10]
		if ALT == "DUP":
			ALT = "INS"
		if ALT not in ["INS", "INV", "DEL", "DUP", "BND"]:
			continue

		info = pase_info(seq[7])
		if len(confbed) > 0:
			if chr not in confbed:
				continue
			if judge_bed(pos, info["END"], confbed[chr]) == 0:
				continue

		if ALT not in callset:
			callset[ALT] = dict()
		if chr not in callset[ALT]:
			if ALT == 'BND':
				callset[ALT][chr] = dict()
			else:
				callset[ALT][chr] = list()

		if ALT == "BND":
			if info["CHR2"] not in callset[ALT][chr]:
				callset[ALT][chr][info["CHR2"]] = list()
			if info["RE"] >= filter:
				callset[ALT][chr][info["CHR2"]].append([pos, info["END"], 0])
		else:
			if info["SVLEN"] >= 50 and info["RE"] >= filter:
				callset[ALT][chr].append([pos, info["SVLEN"], info["END"], 0])
	file.close()
	return callset


def load_callset_sniffles(path, filter, confbed):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		info = pase_info(seq[7])

		if info["SVTYPE"] == "DUP":
			info["SVTYPE"] = "INS"
		svtype = info["SVTYPE"]

		if svtype == "BND":
			chr_2, pos_2 = parse_BND(seq[4])
			if len(confbed) > 0:
				if chr not in confbed:
					continue
				if judge_bed(pos, pos_2, confbed[chr]) == 0:
					continue

			if svtype not in callset:
				callset[svtype] = dict()
			if chr not in callset[svtype]:
				callset[svtype][chr] = dict()
			if chr_2 not in callset[svtype][chr]:
				callset[svtype][chr][chr_2] = list()

			if info["RE"] >= filter:
				callset[svtype][chr][chr_2].append([pos, pos_2, 0])

		else:
			END = info["END"]
			if len(confbed) > 0:
				if chr not in confbed:
					continue
				if judge_bed(pos, END, confbed[chr]) == 0:
					continue

			if svtype not in callset:
				callset[svtype] = dict()
			if chr not in callset[svtype]:
				callset[svtype][chr] = list()
			if info["SVLEN"] >= 50 and info["RE"] >= filter:
				callset[svtype][chr].append([pos, info["SVLEN"], END, 0])

	file.close()
	return callset

def load_callset_svim(path, filter, confbed):
	base_call = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split("\t")
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		ALT = seq[4][1:4]

		if ALT == "DUP":
			ALT = "INS"
		info = pase_info(seq[7])

		if len(confbed) > 0:
			if chr not in confbed:
				continue
			if judge_bed(pos, info["END"], confbed[chr]) == 0:
				continue

		if ALT not in base_call:
			base_call[ALT] = dict()

		if chr not in base_call[ALT]:
			base_call[ALT][chr] = list()

		if ALT == "INV":
			base_call[ALT][chr].append([pos, info["END"] - pos + 1, info["END"], 0])
		else:
			if info["SVLEN"] >= 50:
				base_call[ALT][chr].append([pos, info["SVLEN"], info["END"], 0])
	file.close()
	return base_call

def load_callset_pbsv(path, filter, confbed):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		info = pase_info(seq[7])

		if info["SVTYPE"] not in ["INS", "INV", "DEL", "DUP", "BND"]:
			continue

		try:
			readcount = int(seq[-1].split(':')[1].split(',')[1])
		except:
			continue
		# if readcount < filter:
		# 	continue

		if info["SVTYPE"] == 'BND' and readcount < filter:
			continue

		if info["SVTYPE"] == "DUP":
			info["SVTYPE"] = "INS"

		if info["SVTYPE"] == "BND":
			chr_2, pos_2 = parse_BND(seq[4])
			if len(confbed) > 0:
				if chr not in confbed:
					continue
				if judge_bed(pos, pos_2, confbed[chr]) == 0:
					continue
			if info["SVTYPE"] not in callset:
				callset[info["SVTYPE"]] = dict()
			if chr not in callset[info["SVTYPE"]]:
				callset[info["SVTYPE"]][chr] = dict()
			if chr_2 not in callset[info["SVTYPE"]][chr]:
				callset[info["SVTYPE"]][chr][chr_2] = list()

			callset[info["SVTYPE"]][chr][chr_2].append([pos, pos_2, 0])

		else:
			if info["SVTYPE"] not in callset:
				callset[info["SVTYPE"]] = dict()
			if chr not in callset[info["SVTYPE"]]:
				callset[info["SVTYPE"]][chr] = list()
			if info["SVTYPE"] == "INV":
				info["SVLEN"] = info["END"] - pos + 1
			if len(confbed) > 0:
				if chr not in confbed:
					continue
				if judge_bed(pos, info["END"], confbed[chr]) == 0:
					continue
			if info["SVLEN"] >= 50:
				callset[info["SVTYPE"]][chr].append([pos, info["SVLEN"], info["END"], 0])

	file.close()
	return callset


def eva_record(call_A, call_B, bias, offect):
	for svtype in call_A:
		if svtype not in call_B:
			continue
		else:
			if svtype == "BND":
				for chr1 in call_A[svtype]:
					if chr1 not in call_B[svtype]:
						continue
					else:
						for chr2 in call_A[svtype][chr1]:
							if chr2 not in call_B[svtype][chr1]:
								continue
							else:
								for i in call_A[svtype][chr1][chr2]:
									for j in call_B[svtype][chr1][chr2]:
										if abs(i[0] - j[0]) <= offect and abs(i[1] - j[1]) <= offect:
											i[2] = 1
											j[2] = 1
										else:
											pass
			else:
				for chr in call_A[svtype]:
					if chr not in call_B[svtype]:
						continue
					else:
						for i in call_A[svtype][chr]:
							for j in call_B[svtype][chr]:
								if i[0] - offect <= j[0] <= i[2] + offect or i[0] - offect <= j[2] <= i[2] + offect or j[0] - offect <= i[0] <= j[2] + offect:
									if min(i[1], j[1])*1.0/max(i[1], j[1]) >= bias:
										i[3] = 1
										j[3] = 1
									else:
										pass


def statistics_true_possitive(callset, SVTYPE="ALL"):
	record = 0
	true_record = 0
	if SVTYPE == "ALL":
		for svtype in callset:
			if svtype == "BND":
				for chr1 in callset[svtype]:
					for chr2 in callset[svtype][chr1]:
						for res in callset[svtype][chr1][chr2]:
							record += 1
							if res[2] == 1:
								true_record += 1
			else:
				for chr in callset[svtype]:
					for res in callset[svtype][chr]:
						record += 1
						if res[3] == 1:
							true_record += 1
	else:
		if SVTYPE == "BND":
			for chr1 in callset[SVTYPE]:
				for chr2 in callset[SVTYPE][chr1]:
					for res in callset[SVTYPE][chr1][chr2]:
						record += 1
						if res[2] == 1:
							true_record += 1
		else:
			for chr in callset[SVTYPE]:
				for res in callset[SVTYPE][chr]:
					record += 1
					if res[3] == 1:
						true_record += 1
	return record, true_record


def main_ctrl(args):	
	if args.include == "NULL":
		confbed = dict()
	else:
		logging.info("Load high confidence region.")
		confbed = load_high_conf_bed(args.include)
	logging.info("Load SV callset of selected caller.")
	if args.CALLER == "cuteSV":
		call_child = load_callset_cuteSV(args.F1, int(args.F1_c), confbed)
		call_father = load_callset_cuteSV(args.MP, int(args.MP_c), confbed)
		call_mother = load_callset_cuteSV(args.FP, int(args.FP_c), confbed)
		logging.info("Evaluate accuracy and sensitivity.")
		eva_record(call_child, call_father, args.bias, args.offect)
		eva_record(call_child, call_mother, args.bias, args.offect)
		svtype = ["DEL", "INS", "INV", "BND"]
		for i in svtype:
			child_r, child_tr = statistics_true_possitive(call_child, i)
			father_r, father_tr = statistics_true_possitive(call_father, i)
			mother_r, mother_tr = statistics_true_possitive(call_mother, i)
			accuracy = 1.0*child_tr/child_r
			logging.info("%s\tChild:%d\tFather:%d\tMother:%d."%(i, child_r, father_r, mother_r))
			logging.info("%s: ADI is %.2f."%(i, 100 - 100.0 * accuracy))
	elif args.CALLER == "sniffles":
		call_child = load_callset_sniffles(args.F1, int(args.F1_c), confbed)
		# for svtype in call_child:
		# 	if svtype == "BND":
		# 		for chr in call_child[svtype]:
		# 			for chr_2 in call_child[svtype][chr]:
		# 				for i in call_child[svtype][chr][chr_2]:
		# 					print(svtype, chr, chr_2, i)
		# 	else:
		# 		for chr in call_child[svtype]:
		# 			for i in call_child[svtype][chr]:
		# 				print(svtype, chr, i)
		call_father = load_callset_sniffles(args.MP, int(args.MP_c), confbed)
		call_mother = load_callset_sniffles(args.FP, int(args.FP_c), confbed)
		logging.info("Evaluate accuracy and sensitivity.")
		eva_record(call_child, call_father, args.bias, args.offect)
		eva_record(call_child, call_mother, args.bias, args.offect)
		svtype = ["DEL", "INS", "INV", "BND"]
		for i in svtype:
			child_r, child_tr = statistics_true_possitive(call_child, i)
			father_r, father_tr = statistics_true_possitive(call_father, i)
			mother_r, mother_tr = statistics_true_possitive(call_mother, i)
			accuracy = 1.0*child_tr/child_r
			logging.info("%s\tChild:%d\tFather:%d\tMother:%d."%(i, child_r, father_r, mother_r))
			logging.info("%s: ADI is %.2f."%(i, 100 - 100.0 * accuracy))
	elif args.CALLER == "pbsv":
		call_child = load_callset_pbsv(args.F1, int(args.F1_c), confbed)
		call_father = load_callset_pbsv(args.MP, int(args.MP_c), confbed)
		call_mother = load_callset_pbsv(args.FP, int(args.FP_c), confbed)
		logging.info("Evaluate accuracy and sensitivity.")
		eva_record(call_child, call_father, args.bias, args.offect)
		eva_record(call_child, call_mother, args.bias, args.offect)
		svtype = ["DEL", "INS", "INV", "BND"]
		for i in svtype:
			child_r, child_tr = statistics_true_possitive(call_child, i)
			father_r, father_tr = statistics_true_possitive(call_father, i)
			mother_r, mother_tr = statistics_true_possitive(call_mother, i)
			accuracy = 1.0*child_tr/child_r
			logging.info("%s\tChild:%d\tFather:%d\tMother:%d."%(i, child_r, father_r, mother_r))
			logging.info("%s: ADI is %.2f."%(i, 100 - 100.0 * accuracy))
	elif args.CALLER == 'svim':
		call_child = load_callset_svim(args.F1, int(args.F1_c), confbed)
		call_father = load_callset_svim(args.MP, int(args.MP_c), confbed)
		call_mother = load_callset_svim(args.FP, int(args.FP_c), confbed)
		logging.info("Evaluate accuracy and sensitivity.")
		eva_record(call_child, call_father, args.bias, args.offect)
		eva_record(call_child, call_mother, args.bias, args.offect)
		svtype = ["DEL", "INS", "INV"]
		for i in svtype:
			child_r, child_tr = statistics_true_possitive(call_child, i)
			father_r, father_tr = statistics_true_possitive(call_father, i)
			mother_r, mother_tr = statistics_true_possitive(call_mother, i)
			accuracy = 1.0*child_tr/child_r
			logging.info("%s\tChild:%d\tFather:%d\tMother:%d."%(i, child_r, father_r, mother_r))
			logging.info("%s: ADI is %.2f."%(i, 100 - 100.0 * accuracy))
	child_r, child_tr = statistics_true_possitive(call_child)
	father_r, father_tr = statistics_true_possitive(call_father)
	mother_r, mother_tr = statistics_true_possitive(call_mother)
	accuracy = 1.0*child_tr/child_r
	logging.info("Child:%d\tFather:%d\tMother:%d."%(child_r, father_r, mother_r))
	logging.info("ADI is %.2f."%(100 - 100.0 * accuracy))

def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Evaluate SV callset generated by cuteSV
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="Trio_eval", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("CALLER", type=str, help="Choose a caller")
	parser.add_argument("MP", type=str, help="Male parent callsets")
	parser.add_argument('FP', type=str, help = "Female parent callsets")
	parser.add_argument('F1', type=str, help = "Offspring callsets")
	parser.add_argument("MP_c", type=int, help="Coverage of male parent callsets")
	parser.add_argument('FP_c', type=int, help = "Coverage of female parent callsets")
	parser.add_argument('F1_c', type=int, help = "Coverage of offspring callsets")
	parser.add_argument('-b', '--bias', help = "Bias of overlaping.[%(default)s]", default = 0.7, type = float)
	parser.add_argument('-o', '--offect', help = "Offect of translocation overlaping.[%(default)s]", default = 1000, type = int)
	parser.add_argument('-i', '--include', help = "Include high confident region (bed format).",default = "NULL", type = str)
	# parser.add_argument('-s', '--min_support', help = "Minimum number of reads that support a SV to be reported.[%(default)s]", default = 10, type = int)
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])



''' 
 * All rights Reserved, Designed By HIT-Bioinformatics   
 * @Title:  cuteSV_Description.py
 * @Package: NULL
 * @Description: Descriptions of cuteSV parameters.
 * @author: tjiang
 * @date: May 1st 2019
 * @version V1.0.1   
'''
import argparse

VERSION = '1.0.1'

class cuteSVdp(object):
	'''
	Detailed descriptions of cuteSV version and its parameters.

	'''

	USAGE="""\
	Long read based fast and accurate SV detection with cuteSV.
	
	Current version: v%s
	Author: Tao Jiang
	Contact: tjiang@hit.edu.cn


	Suggestions:

	For PacBio CLR data:
		--max_cluster_bias_INS		100
		--diff_ratio_merging_INS	0.2
		--diff_ratio_filtering_INS	0.6
		--diff_ratio_filtering_DEL	0.7
	For PacBio CCS(HIFI) data:
		--max_cluster_bias_INS		200
		--diff_ratio_merging_INS	0.65
		--diff_ratio_filtering_INS	0.65
		--diff_ratio_filtering_DEL	0.35


	"""%(VERSION)

	MinSizeDel = 'For current version of cuteSV, it can detect deletions larger than this size.'

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="cuteSV", description=cuteSVdp.USAGE, 
		formatter_class=argparse.RawDescriptionHelpFormatter)

	# **************Parameters of input******************
	parser.add_argument("input", 
		metavar="[BAM]", 
		type = str, 
		help ="Sorted .bam file form NGMLR or Minimap2.")
	parser.add_argument('output', 
		type = str, 
		help = "the path of [Output]")
	parser.add_argument('temp_dir', 
		type = str, 
		help = "temporary directory to use for distributed jobs")

	# ************** Other Parameters******************
	parser.add_argument('-t', '--threads', 
		help = "Number of threads to use.[%(default)s]", 
		default = 16, 
		type = int)
	parser.add_argument('-b', '--batches', 
		help = "A batches of reads to load.[%(default)s]", 
		default = 10000000, 
		type = int)
	# The description of batches needs to improve.
	parser.add_argument('-S', '--sample',
		help = "Sample name/id",
		default = "NULL",
		type = str)
	parser.add_argument('-g', '--genotype',
		help = "Enable generate genotype (True/False).[%(default)s]",
		default = "False",
		type = str)

	# **************Parameters in signatures collection******************
	GroupSignaturesCollect = parser.add_argument_group('Collection of SV signatures')
	GroupSignaturesCollect.add_argument('-p', '--max_split_parts', 
		help = "Maximum number of split segments a read may be aligned before it is ignored.[%(default)s]", 
		default = 7, 
		type = int)
	GroupSignaturesCollect.add_argument('-q', '--min_mapq', 
		help = "Minimum mapping quality value of alignment to be taken into account.[%(default)s]", 
		default = 20, 
		type = int)
	GroupSignaturesCollect.add_argument('-r', '--min_read_len', 
		help = "Ignores reads that only report alignments with not longer then bp.[%(default)s]", 
		default = 500, 
		type = int)
	# The min_read_len in last version is 2000.
	# signatures with overlap need to be filtered

	# **************Parameters in clustering******************
	GroupSVCluster = parser.add_argument_group('Generation of SV clusters')
	GroupSVCluster.add_argument('-s', '--min_support', 
		help = "Minimum number of reads that support a SV to be reported.[%(default)s]", 
		default = 3, 
		type = int)
	GroupSVCluster.add_argument('-l', '--min_size', 
		help = "Minimum size of SV to be reported.[%(default)s]", 
		default = 30, 
		type = int)
	GroupSVCluster.add_argument('-L', '--min_siglength', 
		help = "Minimum length of SV signal to be extracted.[%(default)s]", 
		default = 10, 
		type = int)

	# Just a parameter for debug.
	# Will be removed in future.
	# GroupSVCluster.add_argument('--preset',
	# 	help = "Parameter presets for different sequencing technologies (pbclr/pbccs/ont).[%(default)s]",
	# 	default = "pbccs",
	# 	type = str)

	# **************Advanced Parameters******************
	GroupAdvanced = parser.add_argument_group('Advanced')

	# ++++++INS++++++
	GroupAdvanced.add_argument('--max_cluster_bias_INS', 
		help = "Maximum distance to cluster read together for insertion.[%(default)s]", 
		default = 100, 
		type = int)
	GroupAdvanced.add_argument('--diff_ratio_merging_INS', 
		help = "Do not merge breakpoints with basepair identity more than [%(default)s] for insertion.", 
		default = 0.2, 
		type = float)
	GroupAdvanced.add_argument('--diff_ratio_filtering_INS', 
		help = "Filter breakpoints with basepair identity less than [%(default)s] for insertion.", 
		default = 0.6, 
		type = float)

	# ++++++DEL++++++
	GroupAdvanced.add_argument('--max_cluster_bias_DEL', 
		help = "Maximum distance to cluster read together for deletion.[%(default)s]", 
		default = 200, 
		type = int)
	GroupAdvanced.add_argument('--diff_ratio_merging_DEL', 
		help = "Do not merge breakpoints with basepair identity more than [%(default)s] for deletion.", 
		default = 0.3, 
		type = float)
	GroupAdvanced.add_argument('--diff_ratio_filtering_DEL', 
		help = "Filter breakpoints with basepair identity less than [%(default)s] for deletion.", 
		default = 0.7, 
		type = float)

	# ++++++INV++++++
	GroupAdvanced.add_argument('--max_cluster_bias_INV', 
		help = "Maximum distance to cluster read together for inversion.[%(default)s]", 
		default = 500, 
		type = int)

	# ++++++DUP++++++
	GroupAdvanced.add_argument('--max_cluster_bias_DUP', 
		help = "Maximum distance to cluster read together for duplication.[%(default)s]", 
		default = 500, 
		type = int)

	# ++++++TRA++++++
	GroupAdvanced.add_argument('--max_cluster_bias_TRA', 
		help = "Maximum distance to cluster read together for translocation.[%(default)s]", 
		default = 50, 
		type = int)
	GroupAdvanced.add_argument('--diff_ratio_filtering_TRA', 
		help = "Filter breakpoints with basepair identity less than [%(default)s] for translocation.", 
		default = 0.6, 
		type = float)

	# parser.add_argument('-d', '--max_distance', 
	# 	help = "Maximum distance to group SV together..[%(default)s]", 
	# 	default = 1000, type = int)



	# These parameters are drawn lessons from pbsv v2.2.0
	# parser.add_argument('--min_del_size', 
	# 	help = "Minimum size of a deletion.[%(default)s]", 
	# 	default = 20, type = int)

	args = parser.parse_args(argv)
	return args

def Generation_VCF_header(file, contiginfo, sample):
	# General header
	file.write("##fileformat=VCFv4.2\n")
	file.write("##source=cuteSV-%s\n"%(VERSION))
	import time
	file.write("##fileDate=%s\n"%(time.strftime('%Y-%m-%d %H:%M:%S %w-%Z',time.localtime())))
	for i in contiginfo:
		file.write("##contig=<ID=%s,length=%d>\n"%(i[0], i[1]))

	# Specific header
	# ALT
	file.write("##ALT=<ID=INS,Description=\"Insertion of novel sequence relative to the reference\">\n")
	file.write("##ALT=<ID=DEL,Description=\"Deletion relative to the reference\">\n")
	file.write("##ALT=<ID=DUP,Description=\"Region of elevated copy number relative to the reference\">\n")
	file.write("##ALT=<ID=INV,Description=\"Inversion of reference sequence\">\n")
	file.write("##ALT=<ID=TRA,Description=\"Translocation\">\n")

	# INFO
	file.write("##INFO=<ID=PRECISE,Number=0,Type=Flag,Description=\"Precise structural variant\">\n")
	file.write("##INFO=<ID=IMPRECISE,Number=0,Type=Flag,Description=\"Imprecise structural variant\">\n")
	file.write("##INFO=<ID=SVTYPE,Number=1,Type=String,Description=\"Type of structural variant\">\n")
	file.write("##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"Difference in length between REF and ALT alleles\">\n")
	file.write("##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for END coordinate in case of a translocation\">\n")
	file.write("##INFO=<ID=END,Number=1,Type=Integer,Description=\"End position of the variant described in this record\">\n")
	file.write("##INFO=<ID=CIPOS,Number=2,Type=Integer,Description=\"Confidence interval around POS for imprecise variants\">\n")
	file.write("##INFO=<ID=CIEND,Number=2,Type=Integer,Description=\"Confidence interval around END for imprecise variants\">\n")
	file.write("##INFO=<ID=MATEID,Number=.,Type=String,Description=\"ID of mate breakends\">\n")
	file.write("##INFO=<ID=RE,Number=1,Type=Integer,Description=\"Number of read support this record\">\n")

	# FORMAT
	# file.write("\n")
	file.write("##FORMAT=<ID=GT,Number=1,Type=String,Description=\"Genotype\">\n")
	file.write("##FORMAT=<ID=DR,Number=1,Type=Integer,Description=\"# high-quality reference reads\">\n")
	file.write("##FORMAT=<ID=DV,Number=1,Type=Integer,Description=\"# high-quality variant reads\">\n")
	file.write("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t%s\n"%(sample))



# coding=utf-8

from setuptools import setup, find_packages

LONG_DESCRIPTION = '''Long-read sequencing technologies enable to comprehensively \
discover structural variations (SVs). However, it is still non-trivial for \
state-of-the-art approaches to detect SVs with high sensitivity or high performance \
or both. Herein, we propose cuteSV, a sensitive, fast and lightweight SV detection \
approach. cuteSV uses tailored methods to comprehensively collect various types of \
SV signatures, and a clustering-and-refinement method to implement a stepwise SV \
detection, which enables to achieve high sensitivity without loss of accuracy. \
Benchmark results demonstrate that cuteSV has better yields on real datasets. \
Further, its speed and scalability are outstanding and promising to large-scale data analysis.'''

setup(
    name = "cuteSV",
    version = "1.0.1",
    description = "Long read based human genomic structural variation detection with cuteSV",
    author = "Jiang Tao",
    author_email = "tjiang@hit.edu.cn",
    url = "https://github.com/tjiangHIT/cuteSV",
    license = "MIT",
    packages = find_packages("src"),
    package_dir = {"": "src"},
    data_files = [("", ["LICENSE"])],
    scripts=['src/cuteSV/cuteSV'],
    long_description = LONG_DESCRIPTION,
    zip_safe = False,
    install_requires = ['pysam', 'Biopython', 'Cigar', 'numpy']
)



import sys
import numpy as np

def resolution_INV(path, chr, svtype, read_count, max_cluster_bias, sv_size):
	'''
	cluster INV
	************************************************************************
	path:	INV.sigs
	chr:	chromosome id
	svtype:	<INV>
	
	SEQTYPE		read_count 	max_cluster_bias 	sv_size 
	------------------------------------------------------------------------
	CCS			5			10 bp (<500 bp)		20 bp 	
	CLR			5			20 bp (<500 bp)		50 bp 	
	------------------------------------------------------------------------
	
	Input file format
	------------------------------------------------------------------------
	column	#1	#2	#3	#4	#5
			INV	CHR	BP1	BP2	ID	
	#1	inversion type
	#2	chromosome number
	#3	breakpoint_1 in each read
	#4	breakpoint_2 in each read
	#5	read ID
	************************************************************************
	'''

	# Initialization of some temporary variables
	semi_inv_cluster = list()
	semi_inv_cluster.append([0,0,''])
	candidate_single_SV = list()

	# Load inputs & cluster breakpoint from each signature read 
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[1] != chr:
			continue

		breakpoint_1_in_read = int(seq[2])
		breakpoint_2_in_read = int(seq[3])
		read_id = seq[4]

		if breakpoint_1_in_read - semi_inv_cluster[-1][0] > max_cluster_bias:
			if len(semi_inv_cluster) >= read_count:
				generate_semi_inv_cluster_2(semi_inv_cluster, 
											chr, 
											svtype, 
											read_count, 
											sv_size, 
											candidate_single_SV, 
											max_cluster_bias)
			semi_inv_cluster = []
			semi_inv_cluster.append([breakpoint_1_in_read, breakpoint_2_in_read, read_id])
		else:
			semi_inv_cluster.append([breakpoint_1_in_read, breakpoint_2_in_read, read_id])

	if len(semi_inv_cluster) >= read_count:
		generate_semi_inv_cluster_2(semi_inv_cluster, 
									chr, svtype, 
									read_count, 
									sv_size, 
									candidate_single_SV, 
									max_cluster_bias)
	file.close()
	return candidate_single_SV

def generate_semi_inv_cluster(semi_inv_cluster, chr, svtype, read_count, sv_size, candidate_single_SV, max_cluster_bias):

	read_id = [i[2] for i in semi_inv_cluster]
	support_read = len(list(set(read_id)))
	if support_read < read_count:
		return

	breakpoint_1_candidate = [i[0] for i in semi_inv_cluster]
	breakpoint_2_candidate_with_read_id = list()
	for i in semi_inv_cluster:
		breakpoint_2_candidate_with_read_id.append([i[1], i[2]])
	breakpoint_2_candidate = sorted(list(breakpoint_2_candidate_with_read_id), key = lambda x:x[0])

	# print("#")
	# for i in breakpoint_2_candidate:
	# 	print(i)

	breakpoint_1 = np.mean(breakpoint_1_candidate)
	last_bp = breakpoint_2_candidate[0][0]
	temp_count = 1
	max_count = 0
	max_count_id = 0
	temp_sum = last_bp
	max_sum = 0
	temp_id = dict()
	temp_id[breakpoint_2_candidate[0][1]] = 0

	for i in breakpoint_2_candidate[1:]:
		if i[0] - last_bp > max_cluster_bias:
			if temp_count > max_count:
				max_count = temp_count
				max_sum = temp_sum
				max_count_id = len(temp_id)
			temp_id = dict()
			temp_count = 1
			temp_sum = i[0]
			temp_id[i[1]] = 0
		else:
			if i[1] not in temp_id:
				temp_id[i[1]] = 0
			else:
				temp_id[i[1]] += 1
			temp_count += 1
			temp_sum += i[0]
		last_bp = i[0]
	if temp_count > max_count:
		max_count = temp_count
		max_sum = temp_sum
		max_count_id = len(temp_id)

	# print(max_count, len(temp_id), support_read)
	breakpoint_2 = int(max_sum / max_count)
	inv_len = breakpoint_2 - breakpoint_1

	if inv_len >= sv_size and max_count_id >= read_count:
		candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\n'%(chr, svtype, breakpoint_1, breakpoint_2, max_count_id))


def generate_semi_inv_cluster_2(semi_inv_cluster, chr, svtype, read_count, sv_size, candidate_single_SV, max_cluster_bias):

	read_id = [i[2] for i in semi_inv_cluster]
	support_read = len(list(set(read_id)))
	if support_read < read_count:
		return

	inv_cluster_b2 = sorted(semi_inv_cluster, key = lambda x:x[1])


	# breakpoint_1 = np.mean(breakpoint_1_candidate)
	last_bp = inv_cluster_b2[0][1]
	temp_count = 1
	# max_count = 0
	temp_sum_b1 = inv_cluster_b2[0][0]
	temp_sum_b2 = last_bp

	# max_sum = 0
	temp_id = dict()
	temp_id[inv_cluster_b2[0][2]] = 0

	for i in inv_cluster_b2[1:]:
		if i[1] - last_bp > max_cluster_bias:
			if temp_count >= read_count:
				max_count_id = len(temp_id)

				breakpoint_1 = round(temp_sum_b1 / temp_count)
				breakpoint_2 = round(temp_sum_b2 / temp_count)
				inv_len = breakpoint_2 - breakpoint_1
				if inv_len >= sv_size and max_count_id >= read_count:
					# candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\n'%(chr, svtype, breakpoint_1, breakpoint_2, max_count_id))
					candidate_single_SV.append([chr, 
												svtype, 
												str(int(breakpoint_1)), 
												str(int(inv_len)), 
												str(max_count_id)])

			temp_id = dict()
			temp_count = 1
			temp_sum_b1 = i[0]
			temp_sum_b2 = i[1]
			temp_id[i[2]] = 0
		else:
			if i[2] not in temp_id:
				temp_id[i[2]] = 0
			else:
				temp_id[i[2]] += 1
			temp_count += 1
			temp_sum_b1 += i[0]
			temp_sum_b2 += i[1]
		last_bp = i[1]
	if temp_count >= read_count:
		max_count_id = len(temp_id)
		breakpoint_1 = round(temp_sum_b1 / temp_count)
		breakpoint_2 = round(temp_sum_b2 / temp_count)
		inv_len = breakpoint_2 - breakpoint_1
		if inv_len >= sv_size and max_count_id >= read_count:
			# candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\n'%(chr, svtype, breakpoint_1, breakpoint_2, max_count_id))
			candidate_single_SV.append([chr, 
										svtype, 
										str(int(breakpoint_1)), 
										str(int(inv_len)), 
										str(max_count_id)])

def run_inv(args):
	return resolution_INV(*args)